@@ -26,7 +26,7 @@ int BayesSUR_internal(const std::string& dataFile, const std::string& mrfGFile,
                     const std::string& covariancePrior="HIW", 
                     const std::string& gammaPrior="hotspot", const std::string& gammaSampler="bandit", 
                     const std::string& gammaInit = "MLE",
-                    const std::string& betaPrior="independent", const int maxThreads=1,
+                    const std::string& betaPrior="independent", const int maxThreads=1, const int tick=1000,
                     bool output_gamma = true, bool output_beta = true, bool output_Gy = true, bool output_sigmaRho = true, 
                     bool output_pi = true, bool output_tail = true, bool output_model_size = true, bool output_CPO = true, bool output_model_visit = false )
 {
@@ -35,7 +35,7 @@ int BayesSUR_internal(const std::string& dataFile, const std::string& mrfGFile,
   try
   {
     status =  drive(dataFile,mrfGFile,blockFile,structureGraphFile,hyperParFile,outFilePath,nIter,burnin,nChains,
-                    covariancePrior,gammaPrior,gammaSampler,gammaInit,betaPrior,maxThreads,output_gamma, output_beta,
+                    covariancePrior,gammaPrior,gammaSampler,gammaInit,betaPrior,maxThreads,tick,output_gamma, output_beta,
                     output_Gy, output_sigmaRho, output_pi, output_tail, output_model_size, output_CPO, output_model_visit);
   }
   catch(const std::exception& e)
