@@ -499,20 +499,22 @@ arma::mat& HRR_Chain::getBeta() //const
                 {
                     case Beta_Type::gprior :
                     {
-                        W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN,VS_IN) );
+                        //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN,VS_IN) );
+                        W_k = (w+temperature)/(w*temperature) * XtX(VS_IN,VS_IN);
                         break;
                     }
                         
                     case Beta_Type::independent :
                     {
-                        W_k = arma::inv_sympd( XtX(VS_IN,VS_IN)/temperature + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem) );
+                        //W_k = arma::inv_sympd( XtX(VS_IN,VS_IN)/temperature + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem) );
+                        W_k = XtX(VS_IN,VS_IN) + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem);
                         break;
                     }
                         
                     case Beta_Type::reGroup :
                     {
                         //W_k = arma::inv_sympd( XtX(VS_IN,VS_IN)/temperature + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem) );
-                        W_k = arma::inv_sympd( XtX(VS_IN,VS_IN)/temperature + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN.n_elem-nFixedPredictors)) ) );
+                        W_k = XtX(VS_IN,VS_IN) + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN.n_elem-nFixedPredictors)) );
                         break;
                     }
                         
@@ -526,20 +528,22 @@ arma::mat& HRR_Chain::getBeta() //const
                 {
                     case Beta_Type::gprior :
                     {
-                        W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) ) );
+                        //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) ) );
+                        W_k = (w+temperature)/(w*temperature) * data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) );
                         break;
                     }
                         
                     case Beta_Type::independent :
                     {
-                        W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem) );
+                        //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem) );
+                        W_k = data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem);
                         break;
                     }
                         
                     case Beta_Type::reGroup :
                     {
                       // W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN.n_elem,VS_IN.n_elem) );
-                      W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) )/temperature + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN.n_elem-nFixedPredictors)) ) );
+                      W_k = data->cols( (*predictorsIdx)(VS_IN) ).t() * data->cols( (*predictorsIdx)(VS_IN) ) + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN.n_elem-nFixedPredictors)) );
                       break;
                     }
                         
@@ -548,7 +552,8 @@ arma::mat& HRR_Chain::getBeta() //const
                 }
             }
             
-            arma::vec mu_k = W_k * ( data->cols( VS_IN ).t() * data->col( (*outcomesIdx)(k) ) ); // we divide by temp later
+            W_k = arma::inv_sympd( W_k ); // might add settings argument arma::inv_sympd( W_k, inv_opts::allow_approx )
+            arma::vec mu_k = W_k * ( data->cols( VS_IN ).t() * data->col( (*outcomesIdx)(k) ) / temperature ); // we divide by temp later
             
             beta.submat(VS_IN,singleIdx_k) = Distributions::randMvNormal( mu_k , W_k );
         }
@@ -1046,20 +1051,21 @@ double HRR_Chain::logLikelihood( )
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    W_k = (w+temperature)/(w*temperature) * XtX(VS_IN_k,VS_IN_k);
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = XtX(VS_IN_k,VS_IN_k) + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
                 {
                   //W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                  W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                  W_k = XtX(VS_IN_k,VS_IN_k) + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                   break;
                 }
                     
@@ -1073,20 +1079,22 @@ double HRR_Chain::logLikelihood( )
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    W_k = (w+temperature)/(w*temperature) * data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) );
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
                 {
                     //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                    W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                    W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                     break;
                 }
                     
@@ -1095,7 +1103,8 @@ double HRR_Chain::logLikelihood( )
             }
         }
         
-        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k)) ); // we divide by temp later
+        W_k = arma::inv_sympd( W_k );
+        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k)) / temperature ); // we divide by temp later
         //arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) )) ); // we divide by temp later
         
         double a_sigma_k = a_sigma + 0.5*(double)nObservations/temperature;
@@ -1165,20 +1174,22 @@ double HRR_Chain::logLikelihood( const arma::umat&  externalGammaMask )
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    W_k = (w+temperature)/(w*temperature) * XtX(VS_IN_k,VS_IN_k);
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    //W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = XtX(VS_IN_k,VS_IN_k) + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
             {
                 //W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                W_k = XtX(VS_IN_k,VS_IN_k) + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                 break;
             }
                     
@@ -1192,20 +1203,22 @@ double HRR_Chain::logLikelihood( const arma::umat&  externalGammaMask )
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    W_k = (w+temperature)/(w*temperature) * data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) );
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
                 {
                     //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                    W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                    W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                     break;
                 }
                     
@@ -1213,8 +1226,9 @@ double HRR_Chain::logLikelihood( const arma::umat&  externalGammaMask )
                     throw Bad_Beta_Type ( beta_type );
             }
         }
-               
-        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k)) ); // we divide by temp later
+        
+        W_k = arma::inv_sympd( W_k );
+        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k)) / temperature ); // we divide by temp later
         //arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) )) ); // we divide by temp later
         
         double a_sigma_k = a_sigma + 0.5*(double)nObservations/temperature;
@@ -1231,6 +1245,19 @@ double HRR_Chain::logLikelihood( const arma::umat&  externalGammaMask )
         logP += a_sigma*log(b_sigma) - a_sigma_k*log(b_sigma_k);
         
         logP += std::lgamma(a_sigma_k) - std::lgamma(a_sigma);
+        /*
+        if(logP - log(M_PI)*((double)nObservations*(double)nOutcomes*0.5) > 0.){
+            std::cout << "...Debug positive logP=" << logP <<
+            "; k=" << k <<
+            "; beta_type=" << (beta_type == Beta_Type::independent) <<
+            "; tmp=" << tmp <<
+            "; sign=" << sign <<
+            "; w=" << w <<
+            "; a_sigma=" << a_sigma <<
+            "; b_sigma=" << b_sigma <<
+            "; a_sigma_k=" << a_sigma_k <<
+            "; b_sigma_k=" << b_sigma_k << std::endl;
+        } */
     }
     
     logP += -log(M_PI)*((double)nObservations*(double)nOutcomes*0.5); // normalising constant remaining from the likelhood
@@ -1268,20 +1295,22 @@ double HRR_Chain::logLikelihood( arma::umat& externalGammaMask , const arma::uma
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    W_k = (w+temperature)/(w*temperature) * XtX(VS_IN_k,VS_IN_k);
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    //W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = XtX(VS_IN_k,VS_IN_k) + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
                 {
                   //W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                  W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature +  arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                  W_k = XtX(VS_IN_k,VS_IN_k) + arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                   break;
                 }
                     
@@ -1295,20 +1324,22 @@ double HRR_Chain::logLikelihood( arma::umat& externalGammaMask , const arma::uma
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    //W_k = (w*temperature)/(w+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    W_k = (w+temperature)/(w*temperature) * data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) );
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
                 {
                   //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./w * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                  W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature +  arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                  W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) +  arma::diagmat( arma::join_cols(1./w0*arma::ones(nFixedPredictors),1./w*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                   break;
                 }
                     
@@ -1316,8 +1347,9 @@ double HRR_Chain::logLikelihood( arma::umat& externalGammaMask , const arma::uma
                     throw Bad_Beta_Type ( beta_type );
             }
         }
-               
-        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k)) ); // we divide by temp later
+        
+        W_k = arma::inv_sympd( W_k );
+        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k))/temperature ); // we divide by temp later
         //arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) )) ); // we divide by temp later
         
         double a_sigma_k = a_sigma + 0.5*(double)nObservations/temperature;
@@ -1370,20 +1402,21 @@ double HRR_Chain::logLikelihood( const arma::umat& externalGammaMask , const dou
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (externalW*temperature)/(externalW+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    //W_k = (externalW*temperature)/(externalW+temperature) * arma::inv_sympd( XtX(VS_IN_k,VS_IN_k) );
+                    W_k = (externalW+temperature)/(externalW*temperature) * XtX(VS_IN_k,VS_IN_k);
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./externalW * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = XtX(VS_IN_k,VS_IN_k) + 1./externalW * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
                 {
                     // W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + 1./externalW * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                    W_k = arma::inv_sympd( XtX(VS_IN_k,VS_IN_k)/temperature + arma::diagmat( arma::join_cols(1./externalW0*arma::ones(nFixedPredictors),1./externalW*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                    W_k = XtX(VS_IN_k,VS_IN_k) + arma::diagmat( arma::join_cols(1./externalW0*arma::ones(nFixedPredictors),1./externalW*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                     break;
                 }
                     
@@ -1397,20 +1430,21 @@ double HRR_Chain::logLikelihood( const arma::umat& externalGammaMask , const dou
             {
                 case Beta_Type::gprior :
                 {
-                    W_k = (externalW*temperature)/(externalW+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    //W_k = (externalW*temperature)/(externalW+temperature) * arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) ) );
+                    W_k = (externalW+temperature)/(externalW*temperature) * data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) );
                     break;
                 }
                     
                 case Beta_Type::independent :
                 {
-                    W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./externalW * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
+                    W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) + 1./externalW * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem);
                     break;
                 }
                     
                 case Beta_Type::reGroup :
                 {
                   //W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + 1./externalW * arma::eye<arma::mat>(VS_IN_k.n_elem,VS_IN_k.n_elem) );
-                  W_k = arma::inv_sympd( ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) )/temperature + arma::diagmat( arma::join_cols(1./externalW0*arma::ones(nFixedPredictors),1./externalW*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) ) );
+                  W_k = data->cols( (*predictorsIdx)(VS_IN_k) ).t() * data->cols( (*predictorsIdx)(VS_IN_k) ) + arma::diagmat( arma::join_cols(1./externalW0*arma::ones(nFixedPredictors),1./externalW*arma::ones(VS_IN_k.n_elem-nFixedPredictors)) );
                   break;
                 }
                     
@@ -1418,8 +1452,9 @@ double HRR_Chain::logLikelihood( const arma::umat& externalGammaMask , const dou
                     throw Bad_Beta_Type ( beta_type );
             }
         }
-               
-        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k)) ); // we divide by temp later
+        
+        W_k = arma::inv_sympd( W_k );
+        arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) ) - yMean.col(k)) / temperature ); // we divide by temp later
         //arma::vec mu_k = W_k * ( data->cols( (*predictorsIdx)(VS_IN_k) ).t() * (data->col( (*outcomesIdx)(k) )) ); // we divide by temp later
         
         double a_sigma_k = externalA_sigma + 0.5*(double)nObservations/temperature;
@@ -1718,6 +1753,7 @@ void HRR_Chain::stepPi()
     
 }
 
+// MH sampling for slab's variance w, also for random effects' w0 if needed
 void HRR_Chain::stepW()
 {
     double proposedW = std::exp( std::log(w) + randNormal(0.0, var_w_proposal) );
@@ -1735,31 +1771,49 @@ void HRR_Chain::stepW()
         
         ++w_acc_count;
     }
-}
+    
+    // update w0 if there are random effects
+    if( nFixedPredictors > 0 )
+    {
+        double proposedW0 = std::exp( std::log(w0) + randNormal(0.0, var_w0_proposal) );
+        double proposedW0Prior = logPW0( proposedW0 );
+        proposedLikelihood = logLikelihood( gammaMask , w , proposedW0 , a_sigma , b_sigma );
 
+        double logAccProb = (proposedW0Prior + proposedLikelihood) - (logP_w0 + log_likelihood);
+
+        if( randLogU01() < logAccProb )
+          {
+            w0 = proposedW0;
+            logP_w0 = proposedW0Prior;
+            log_likelihood = proposedLikelihood;
+          }
+    }
+}
+/*
 void HRR_Chain::stepW0()
 {
-  double proposedW = std::exp( std::log(w) + randNormal(0.0, var_w_proposal) );
+  //double proposedW = std::exp( std::log(w) + randNormal(0.0, var_w_proposal) ); // no need to sample a new w
   double proposedW0 = std::exp( std::log(w0) + randNormal(0.0, var_w0_proposal) );
 
-  double proposedWPrior = logPW( proposedW );
+  //double proposedWPrior = logPW( proposedW );
   double proposedW0Prior = logPW0( proposedW0 );
-  double proposedLikelihood = logLikelihood( gammaMask , proposedW , proposedW0 , a_sigma , b_sigma );
+  double proposedLikelihood = logLikelihood( gammaMask , w , proposedW0 , a_sigma , b_sigma );
 
-  double logAccProb = (proposedWPrior + proposedW0Prior + proposedLikelihood) - (logP_w + logP_w0 + log_likelihood);
+  //double logAccProb = (proposedWPrior + proposedW0Prior + proposedLikelihood) - (logP_w + logP_w0 + log_likelihood);
+  double logAccProb = (proposedW0Prior + proposedLikelihood) - (logP_w0 + log_likelihood);
 
   if( randLogU01() < logAccProb )
     {
-      w = proposedW;
+      //w = proposedW;
       w0 = proposedW0;
-      logP_w = proposedWPrior;
+      //logP_w = proposedWPrior;
       logP_w0 = proposedW0Prior;
       log_likelihood = proposedLikelihood;
 
       ++w_acc_count;
       //++w0_acc_count;
     }
-}
+} */
 
 void HRR_Chain::stepGamma()
 {
@@ -1808,7 +1862,7 @@ void HRR_Chain::stepGamma()
         gamma_acc_count += 1.; // / updatedOutcomesIdx.n_elem;
     }
     
-    updateGammaMask();
+    //updateGammaMask();
     
     // after A/R, update bandit Related variables
     if( gamma_sampler_type == Gamma_Sampler_Type::bandit )
@@ -2270,13 +2324,13 @@ void HRR_Chain::swapAll( std::shared_ptr<HRR_Chain>& thatChain )
     
     // HARD SWAP cause swapping "this" is not an option
     // swap quantities
-    //arma::umat swapGammaMask;
+    arma::umat swapGammaMask;
     arma::mat swapMat;
     
-    /*swapGammaMask = this->getGammaMask() ;
+    swapGammaMask = this->getGammaMask() ;
     this->setGammaMask( thatChain->getGammaMask() );
-    thatChain->setGammaMask( swapGammaMask );*/
-    this->gammaMask.swap(thatChain->gammaMask);
+    thatChain->setGammaMask( swapGammaMask );
+    //this->gammaMask.swap(thatChain->gammaMask); // Not sure if function swap() sloves an swapping issue?
     
     // parameters and priors
     if ( gamma_type == Gamma_Type::hotspot )
@@ -2303,7 +2357,7 @@ void HRR_Chain::swapAll( std::shared_ptr<HRR_Chain>& thatChain )
 int HRR_Chain::globalStep( std::shared_ptr<HRR_Chain>& that )
 {
     
-    unsigned int globalType = randIntUniform(0,3);
+    unsigned int globalType = randIntUniform(0,5);
     
     switch(globalType){
             
@@ -2324,6 +2378,10 @@ int HRR_Chain::globalStep( std::shared_ptr<HRR_Chain>& that )
             return this -> block_crossOver_step( that , corrMatX , 0.25 );
             break;
             
+        case 4:
+            return this -> exchangeAll_step( that );
+            break;
+            
         default:
             break;
     }
@@ -2331,6 +2389,25 @@ int HRR_Chain::globalStep( std::shared_ptr<HRR_Chain>& that )
     return 0;
 }
 
+int HRR_Chain::exchangeAll_step( std::shared_ptr<HRR_Chain>& thatChain )
+{
+    
+    double logPExchange = ( this->getLogLikelihood() * this->getTemperature() -
+                           thatChain->getLogLikelihood() * thatChain->getTemperature() ) *
+    ( 1. / thatChain->getTemperature() - 1. / this->getTemperature() );
+    //  no priors because that is not tempered so it cancels out
+    
+    if( randLogU01() < logPExchange )
+    {
+        // Swap all the states
+        this -> swapAll( thatChain );
+        
+        return 1;
+    }else
+        return 0;
+}
+
+
 // *******************************
 // Other Methods
 // *******************************
